// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
syntax = "proto2";
package kudu;

option java_package = "org.apache.kudu";

import "google/protobuf/descriptor.proto";

// ============================================================================
//  Protobuf container metadata
// ============================================================================

// Supplemental protobuf container header, after the main header (see
// pb_util.h for details).
message ContainerSupHeaderPB {
  // The protobuf schema for the messages expected in this container.
  //
  // This schema is complete, that is, it includes all of its dependencies
  // (i.e. other schemas defined in .proto files imported by this schema's
  // .proto file).
  required google.protobuf.FileDescriptorSet protos = 1;

  // The PB message type expected in each data entry in this container. Must
  // be fully qualified (i.e. kudu.tablet.TabletSuperBlockPB).
  required string pb_type = 2;
}

extend google.protobuf.FieldOptions {
  optional bool REDACT = 50001 [default=false];
}