syntax = "proto3";
option java_package = "com.clickzetta.jobcache.proto";
option java_outer_classname = "JobCacheProtos";
package com.clickzetta.jobcache;
option java_multiple_files = true;


message JobResultCacheMeta {
  optional uint64 instance_id = 1;
  optional uint64 workspace_id = 2;
  optional string job_signature = 3;
  optional string preprocessed_plan = 4;
  optional string result_files = 5;
  optional uint64 start_time_ms = 6;
}